//==========================================================================
// File: util.c
// Description: Some utility function used by routing algorithms
// Author: Pat Morin
// Created: 
//==========================================================================
#include <LEDA/ray.h>
#include <LEDA/line.h>

#include "util.h"


edge find_first(const graph &G, const node_array<point> &pos,  
		node s, node t)
{
  edge e, emax = nil;
  double alphamax = -HUGE_VAL;
  forall_adj_edges (e, s) {
    double a1 = ray(pos[s], pos[t]).direction();
    double a2 = ray(pos[s], pos[G.target(e)]).direction();
    double alpha;
    if (a2 > a1) {
      alpha = a2 - a1;
    } else {
      alpha = a2 + PI_TIMES_2 - a1;
    }
    if (alpha == 0.0) alpha = PI_TIMES_2;
    if (alpha > alphamax) {
      alphamax = alpha;
      emax = e;
    }
  }
  return emax;
}


bool valid_path(const graph &G, const list<edge> &P, node s, node t)
{
  if (P.empty())
    return s == t;

  list_item it = P.first();
  if (G.source(P[it]) != s)
    return false;
  for (it = P.succ(it); it != nil; it = P.succ(it))
    if (G.target(P[P.pred(it)]) != G.source(P[it]))
      return false;
  return G.target(P[P.last()]) == t;
}


int robust_intersection_test(const SEGMENT &s, const SEGMENT &e, POINT &p)
{
  REAL sd = s.sqr_dist(e.source());
  REAL td = s.sqr_dist(e.target());

  if (sd < EPSILON_SQUARED) {
    if (td < EPSILON_SQUARED) {
      return I_OVERLAP;
    }
    p = e.source();
    return I_SOURCE;
  }
  if (td < EPSILON_SQUARED) {
    p = e.target();
    return I_TARGET;
  }
  if (s.intersection(e, p))
    return I_INTERIOR;
  return I_NONE;
}


int robust_distance_test(const SEGMENT &e, const POINT &p, POINT &px)
{
  REAL ds = e.source().sqr_dist(p);
  REAL dt = e.target().sqr_dist(p);
  REAL di = e.sqr_dist(p);

  if (ds - di < EPSILON_SQUARED) {
    px = e.source();
    return D_SOURCE;
  }
  if (dt - di < EPSILON_SQUARED) {
    px = e.target();
    return D_TARGET;
  }
  line l(e);
  line(l.perpendicular(p)).intersection(l, px);
  return D_INTERIOR;
}
