//==========================================================================
// File: util.h
// Description: Some utility function used by routing algorithms
// Author: Pat Morin
// Created: 
//==========================================================================
#ifndef __UTIL_H
#define __UTIL_H

#include <LEDA/graph.h>
#include <LEDA/point.h>
#include <LEDA/segment.h>
#include <LEDA/ray.h>
#include <LEDA/node_array.h>

// typedefs in case we ever want to switch to exact arithmetic
typedef point POINT;
typedef segment SEGMENT;
typedef ray RAY;
typedef double REAL;

// pi * 2
#define PI_TIMES_2 (3.1415926535897932384626433832795028841972*2.0)

// two points at distance <= EPSILON from each other are considered
// equal
#define EPSILON 0.00001
#define EPSILON_SQUARED (EPSILON*EPSILON)

// return values for intersection_test
enum {I_SOURCE, I_TARGET, I_INTERIOR, I_OVERLAP, I_NONE};

// return values for distance_test
enum {D_SOURCE, D_TARGET, D_INTERIOR};


edge find_first(const graph &G, const node_array<POINT> &pos,  
		node s, node t);


bool valid_path(const graph &G, const list<edge> &P, node s, node t);

int robust_intersection_test(const SEGMENT &s, const SEGMENT &e, POINT &p);


int robust_distance_test(const SEGMENT &e, const POINT &p, POINT &px);


#endif // __UTIL_H








